#!/bin/bash

function die(){
    echo "Invalid parameters"
    usage
    exit 1
}

function read_backlight(){
    local rtrn=1
    local times=0
    local msg=""

    msg=$(ddcutil getvcp 10 2> /dev/null)
    rtrn=$?
    while [[ $rtrn -ne 0 && "$times" -lt 3 ]];do
        msg=$(ddcutil getvcp 10 2> /dev/null)
        rtrn=$?
        let times++
    done

    if [ "$rtrn" -ne 0 ];then
        echo "Read failed"
    else
        msg=$(echo $msg | awk '{print $9}' | sed 's/,//')
        echo "backlight: $msg"
    fi

    return $rtrn
    
}
function set_backlight(){
    local value=$1
    local rtrn=1
    local times=0

    if [ "${value:0:2}" == "0x" ]; then
        value=$((value))
    fi

    [[ "$value" =~ ^[0-9]+$ ]] || die

    if [ "$value" -gt 100 ];then
        value=100
    fi

    [ "$value" -le 100 ] && [ "$value" -ge 0 ] || die

    ddcutil setvcp 0x10 "$value" > /dev/null 2>&1
    rtrn=$?
    while [[ $rtrn -ne 0 && "$times" -lt 3 ]];do
        ddcutil setvcp 0x10 "$value" > /dev/null 2>&1
        rtrn=$?
        let times++
    done

    if [ $rtrn -eq 0 ];then
        echo "Set backlight to $value"
    else
        echo "Failed to set backlight"
    fi

    return $rtrn
}

function usage(){
    local name=$(basename $0)
    echo "Usage: $name value, value range 0-100"
    echo "Example: $name 100"
    echo " -r: Read backlight value"
    echo " -h: Usage"
}

which ddcutil > /dev/null
if [ $? -ne 0 ];then
    echo "Not found ddcutil command"
    echo "Please install ddcutil via apt"
    echo "sudo apt update && sudo apt install -y ddcutil"
    exit 1
fi

while getopts "rh" option;do
    case "${option}" in
    r) read_backlight
        exit $?
      ;;
    h) usage
        exit 0
      ;;
    *) echo "Unknown argument \"${option}\""
      die
      ;;
    esac
done

[ $1 ] || exit 1
set_backlight "$1"